package main

import (
	"flag"
	"fmt"
	mapset "github.com/deckarep/golang-set"
	"pestiot/solutions"
	"pestiot/utils"
	"strings"
	"time"
)

// 命令行接收参数
var (
	provIds      string
	beginDate    string
	endDate      string
	devIds       string
	solution     string
	fileDir      string
	mentioneds   string
	threadNumIO  int
	threadNumNet int
)

// 获取命令行参数
func getCmdParams() bool {
	flag.StringVar(&provIds, "provIds", "", "省份编号(必填)(e.g., 650000,340000)")
	flag.StringVar(&beginDate, "beginDate", "", "开始时间(必填)(e.g., 2022-07-01)")
	flag.StringVar(&endDate, "endDate", "", "结束时间(必填)(e.g., 2022-09-30)")
	flag.StringVar(&devIds, "devIds", "", "设备编号(可选)(e.g., 129301230102,12039912093,21399422244)")
	flag.StringVar(&solution, "solution", "", "解决方案(必填)(e.g., 01)\n01 计算设备可用率\n02 灯诱及性诱日统计\n03 本地图片核查\n04 目录下文件数量统计")
	flag.StringVar(&fileDir, "fileDir", "", "文件路径(可选)(e.g., D:/iotImage)")
	flag.IntVar(&threadNumIO, "threadNumIO", 100, "IO操作并发协程数量(可选)(e.g., 100)")
	flag.IntVar(&threadNumNet, "threadNumNet", 10, "Net操作并发协程数量(可选)(e.g., 10)")
	flag.StringVar(&mentioneds, "mentioneds", "", "消息提醒人员(可选)(e.g., gpguo@jinhetech.com,eszhong@jinhetech.com)")
	flag.Parse()
	if solution == "" {
		fmt.Println("必要参数(solution)缺失,请使用 -help 命令查看入参！")
		return false
	}
	switch solution {
	case "01":
	case "02":
		if provIds == "" || beginDate == "" || endDate == "" {
			fmt.Println("必要参数(provIds beginDate endDate)缺失,请使用 -help 命令查看入参！")
			return false
		}
		break
	case "03":
		if provIds == "" || beginDate == "" || endDate == "" || fileDir == "" {
			fmt.Println("必要参数(provIds beginDate endDate fileDir)缺失,请使用 -help 命令查看入参！")
			return false
		}
		break
	case "04":
		if fileDir == "" {
			fmt.Println("必要参数(fileDir)缺失,请使用 -help 命令查看入参！")
			return false
		}
		break
	default:
		fmt.Println("无效的参数(solution),请使用 -help 命令查看入参！")
		return false
	}
	return true
}

func main() {
	start := time.Now().Unix()

	if !getCmdParams() {
		return
	}

	devSet := mapset.NewSet()
	if devIds != "" {
		for _, v := range strings.Split(devIds, ",") {
			devSet.Add(v)
		}
	}
	var mentionedList []string
	if mentioneds != "" {
		for _, v := range strings.Split(mentioneds, ",") {
			mentionedList = append(mentionedList, v)
		}
	}
	provIdList := strings.Split(provIds, ",")
	var fileList [][]string
	var dataCount int
	for _, provId := range provIdList {
		prov := provId
		var filepath, msg string
		if solution == "01" { // 计算设备可用率
			filepath, msg = solutions.CountEquipmentAvailability(prov, beginDate, endDate, devSet)
		} else if solution == "02" {
			filepath, msg = solutions.LightAndSexLureStatistics(prov, beginDate, endDate)
		} else if solution == "03" {
			var count int
			filepath, msg, count = solutions.LocalImageFileCheck(prov, beginDate, endDate, fileDir, threadNumIO, threadNumNet)
			dataCount += count
		} else if solution == "04" {
			solutions.CountFileNum(fileDir)
		}
		if filepath != "" {
			fileList = append(fileList, []string{filepath, msg})
		}
	}
	for _, itm := range fileList {
		utils.WechatFileUpload(itm[0], itm[1], &mentionedList, nil)
		time.Sleep(1 * time.Second)
	}

	end := time.Now().Unix()
	fmt.Printf("执行结束,共计耗时%dmin,数据量%d\n", (end-start)/60, dataCount)
}
