package solutions

import (
	"database/sql"
	"fmt"
	mapset "github.com/deckarep/golang-set"
	_ "github.com/denisenkom/go-mssqldb"
	"github.com/xuri/excelize/v2"
	"pestiot/eneity"
	"pestiot/static"
	"pestiot/utils"
	"time"
)

// 表头
var cols = []eneity.Column{
	{"城市名", "CityName"},
	{"县名", "AreaName"},
	{"账号名", "DeptShort"},
	{"监测点名称", "SiteName"},
	{"厂商名称", "FactName"},
	{"设备类型", "DevTypeName"},
	{"设备名称", "DevName"},
	{"设备编号", "DevID"},
	{"设备状态", "DevStatusName"},
	{"厂商编号", "FactDev"},
	{"厂商编号", "DevCode"},
}

func CountEquipmentAvailability(provId, beginDate, endDate string, devSet mapset.Set) (string, string) {
	dateList := utils.GetBetweenDates(beginDate, endDate)
	columnList := cols
	for _, v := range dateList {
		columnList = append(columnList, eneity.Column{Name: v, Prop: v})
	}

	dbInfo := &eneity.DbInfo{
		Server:   static.Server,
		User:     static.User,
		Password: static.Password,
		Port:     int32(static.Port),
		Database: static.Database,
		Dbtype:   static.Dbtype,
	}
	db, err := dbInfo.GetConnect()
	if err != nil {
		fmt.Println("sql连接异常:", err)
		return "", ""
	}
	sqllan := fmt.Sprintf(
		"select t1.DevID, t1.DevName, t2.DeptShort, t2.CityName, t2.AreaName, t3.SiteName, t4.DevTypeName, t5.FactName, t6.FactDev, t6.DevCode, t7.DevStatusName from T_Bas_Dev t1 left join T_Con_Dev t6 on t1.DevID = t6.DevID, T_Bas_Dept t2, T_Bas_Site t3, T_Pmt_Dev_Type t4, T_Bas_Factory t5, T_Pmt_Dev_status t7 where t1.DeptID = t2.DeptID and t1.MonSite = t3.MonSite and t1.DevType = t4.DevType and t1.FactID = t5.FactID and t1.ProvID = '%s' and t1.DevStatus = t7.DevStatus order by DevID",
		provId)
	fmt.Println(sqllan)
	rows, err := db.Query(sqllan)
	if err != nil {
		fmt.Println("获取设备数据异常:", err)
		return "", ""
	}
	var devList []string
	dataMap := make(map[string]map[string]string)
	dataCountMap := make(map[string]int32)
	days := len(dateList)
	for rows.Next() {
		var (
			CityName      sql.NullString
			AreaName      sql.NullString
			DeptShort     sql.NullString
			SiteName      sql.NullString
			FactName      sql.NullString
			DevTypeName   sql.NullString
			DevName       sql.NullString
			DevID         sql.NullString
			DevStatusName sql.NullString
			FactDev       sql.NullString
			DevCode       sql.NullString
		)
		err := rows.Scan(&DevID, &DevName, &DeptShort, &CityName, &AreaName, &SiteName, &DevTypeName, &FactName, &FactDev, &DevCode, &DevStatusName)
		if err != nil {
			fmt.Println("读取设备数据异常异常:", err)
			return "", ""
		}
		if DevID.String == "" {
			continue
		}
		if devSet.Cardinality() > 0 && !devSet.Contains(DevID.String) {
			continue
		}
		devList = append(devList, DevID.String)
		devInfo := make(map[string]string)
		devInfo["DevID"] = DevID.String
		devInfo["DevName"] = DevName.String
		devInfo["DeptShort"] = DeptShort.String
		devInfo["CityName"] = CityName.String
		devInfo["AreaName"] = AreaName.String
		devInfo["SiteName"] = SiteName.String
		devInfo["DevTypeName"] = DevTypeName.String
		devInfo["FactName"] = FactName.String
		devInfo["FactDev"] = FactDev.String
		devInfo["DevCode"] = DevCode.String
		devInfo["DevStatusName"] = DevStatusName.String
		dataMap[DevID.String] = devInfo
	}
	sqllan = fmt.Sprintf("select DevID, DataYMD, sum(num) Num from (select DevID, DataYMD, count(*) num from T_Dat_Detail_%s WITH(NOLOCK) where DataYMD >= '%s' and DataYMD <= '%s' group by DevID, DataYMD union all select DevID, DataYMD, count(*) num from T_Dat_Image_%s WITH(NOLOCK) where DataYMD >= '%s' and DataYMD <= '%s' group by DevID, DataYMD)a group by DevID, DataYMD", provId, dateList[0], dateList[days-1], provId, dateList[0], dateList[days-1])
	fmt.Println(sqllan)
	rows, err = db.Query(sqllan)
	if err != nil {
		fmt.Println("获取设备统计数据异常:", err)
		return "", ""
	} else {
		fmt.Println("获取到数据")
	}
	for rows.Next() {
		var (
			DevID   sql.NullString
			DataYMD sql.NullString
			Num     sql.NullInt32
		)
		err := rows.Scan(&DevID, &DataYMD, &Num)
		if err != nil {
			fmt.Println("读取设备统计数据异常:", err)
			return "", ""
		}
		devId := DevID.String
		data, ok := dataMap[devId]
		if !ok {
			continue
		}
		data[DataYMD.String] = fmt.Sprintf("%d", Num.Int32)

		if dataCount, ok := dataCountMap[devId]; ok {
			dataCountMap[devId] = dataCount + 1
		} else {
			dataCountMap[devId] = 1
		}
	}

	xlsx := excelize.NewFile()
	var header []interface{}
	for _, col := range columnList {
		header = append(header, col.Name)
	}
	header = append(header, "设备数据联通率(%)")
	err = xlsx.SetSheetRow("Sheet1", "A1", &header)
	if err != nil {
		fmt.Println("插入表头异常:", err)
		return "", ""
	}
	for idx, devId := range devList {
		var info []interface{}
		data := dataMap[devId]
		for _, col := range columnList {
			if v, ok := data[col.Prop]; ok {
				info = append(info, v)
			} else {
				info = append(info, "0")
			}
		}
		if data["DevTypeName"] == "视频监控" {
			info = append(info, "/")
		} else {
			if num, ok := dataCountMap[devId]; ok {
				info = append(info, fmt.Sprintf("%.2f", 100*float64(num)/float64(days)))
			} else {
				info = append(info, "0.00")
			}
		}
		row := fmt.Sprintf("A%d", idx+2)
		err := xlsx.SetSheetRow("Sheet1", row, &info)
		if err != nil {
			fmt.Println(fmt.Sprintf("插入行%s异常:", row), err)
		}
	}
	filepath := fmt.Sprintf("./file/[result]%s设备(%s~%s)可用率统计结果_%d.xlsx", static.Citys[provId], beginDate, endDate, time.Now().Unix())
	err = xlsx.SaveAs(filepath)
	if err != nil {
		fmt.Println("保存excel异常:", err)
		return "", ""
	}
	err = db.Close()
	if err != nil {
		fmt.Println("数据库连接关闭异常:", err)
		return "", ""
	}
	msg := fmt.Sprintf("%s设备(%s~%s)可用率统计结果可见附件，请尽快完成设备排查并完成数据补录。", static.Citys[provId], beginDate, endDate)
	return filepath, msg
}
