package solutions

import (
	"database/sql"
	"fmt"
	mapset "github.com/deckarep/golang-set"
	"github.com/xuri/excelize/v2"
	"pestiot/eneity"
	"pestiot/static"
	"pestiot/utils"
	"time"
)

func LightAndSexLureStatistics(provId, beginDate, endDate string) (string, string) {
	dateList := utils.GetBetweenDates(beginDate, endDate)
	days := len(dateList)

	dbInfo := &eneity.DbInfo{Server: static.Server, User: static.User, Password: static.Password, Port: int32(static.Port), Database: static.Database, Dbtype: static.Dbtype}
	db, err := dbInfo.GetConnect()
	if err != nil {
		fmt.Println("sql连接异常:", err)
		return "", ""
	}
	// 获取设备信息
	sqllan := fmt.Sprintf(
		"select t1.DevID, t1.DevName, t1.SourceID, t2.CityName, t2.AreaName, t3.SiteName, t4.DevTypeName from T_Bas_Dev t1, T_Bas_Dept t2, T_Bas_Site t3, T_Pmt_Dev_Type t4 where t1.DeptID = t2.DeptID and t1.MonSite = t3.MonSite and t1.DevType = t4.DevType and t1.ProvID = '%s' order by DevID",
		provId)
	rows, err := db.Query(sqllan)
	if err != nil {
		fmt.Println("获取设备数据异常:", err)
		return "", ""
	}
	var lightDevList []map[string]string
	var sexDevList []map[string]string
	for rows.Next() {
		var (
			CityName    sql.NullString
			AreaName    sql.NullString
			SiteName    sql.NullString
			DevTypeName sql.NullString
			DevName     sql.NullString
			DevID       sql.NullString
			SourceID    sql.NullString
		)
		err = rows.Scan(&DevID, &DevName, &SourceID, &CityName, &AreaName, &SiteName, &DevTypeName)
		if err != nil {
			fmt.Println("读取设备数据异常异常:", err)
			return "", ""
		}
		devInfo := make(map[string]string)
		devInfo["DevID"] = DevID.String
		devInfo["DevName"] = DevName.String
		devInfo["CityName"] = CityName.String
		devInfo["AreaName"] = AreaName.String
		devInfo["SiteName"] = SiteName.String
		devInfo["DevTypeName"] = DevTypeName.String
		devInfo["SourceID"] = SourceID.String
		if DevTypeName.String == "虫情测报灯" {
			lightDevList = append(lightDevList, devInfo)
		} else if DevTypeName.String == "性诱监测仪" {
			sexDevList = append(sexDevList, devInfo)
		}
	}
	// 获取灯诱数据
	sqllan = fmt.Sprintf(
		"select a.DevID, a.DataYMD, b.MonItemName, a.DevCount from (select a.DevID, a.DataYMD, b.MonItem, sum(b.DevCount) DevCount from T_Dat_Image_%s a, T_Dat_Pest_Count b, T_Bas_Dev c where a.DevID = c.DevID and c.DevType = '07' and a.DataYMD >= '%s' and a.DataYMD <= '%s' and a.ImageCode = b.ImageUrl group by a.DevID, a.DataYMD, b.MonItem)a, T_Std_Item b where a.MonItem = b.MonItem",
		provId, dateList[0], dateList[days-1])
	rows, err = db.Query(sqllan)
	if err != nil {
		fmt.Println("获取灯诱数据异常:", err)
		return "", ""
	}
	lightDevDateDataMap := make(map[string]map[string]map[string]float64)
	lightDateDataMap := make(map[string]map[string]float64)
	lightMonitemSet := mapset.NewSet()
	for rows.Next() {
		var (
			DevID       sql.NullString
			DataYMD     sql.NullString
			MonItemName sql.NullString
			DevCount    sql.NullFloat64
		)
		err = rows.Scan(&DevID, &DataYMD, &MonItemName, &DevCount)
		if err != nil {
			fmt.Println("读取灯诱数据异常异常:", err)
			return "", ""
		}
		if _, ok := lightDevDateDataMap[DevID.String]; !ok {
			lightDevDateDataMap[DevID.String] = make(map[string]map[string]float64)
		}
		if _, ok := lightDevDateDataMap[DevID.String][DataYMD.String]; !ok {
			lightDevDateDataMap[DevID.String][DataYMD.String] = make(map[string]float64)
			lightDevDateDataMap[DevID.String][DataYMD.String]["sum"] = 0
		}
		if _, ok := lightDevDateDataMap[DevID.String][DataYMD.String][MonItemName.String]; !ok {
			lightDevDateDataMap[DevID.String][DataYMD.String][MonItemName.String] = 0
		}
		lightDevDateDataMap[DevID.String][DataYMD.String][MonItemName.String] += DevCount.Float64
		lightDevDateDataMap[DevID.String][DataYMD.String]["sum"] += DevCount.Float64

		if _, ok := lightDateDataMap[DataYMD.String]; !ok {
			lightDateDataMap[DataYMD.String] = make(map[string]float64)
		}
		if _, ok := lightDateDataMap[DataYMD.String][MonItemName.String]; !ok {
			lightDateDataMap[DataYMD.String][MonItemName.String] = 0
			lightDateDataMap[DataYMD.String]["sum"] = 0
		}
		lightDateDataMap[DataYMD.String][MonItemName.String] += DevCount.Float64
		lightDateDataMap[DataYMD.String]["sum"] += DevCount.Float64
		lightMonitemSet.Add(MonItemName.String)
	}
	// 获取性诱数据
	sqllan = fmt.Sprintf(
		"select a.DevID, a.DataYMD, c.MonItemName, sum(a.DataValue) DevCount from T_Dat_Detail_%s a, T_Bas_Dev b, T_Std_Item c where a.DevID = b.DevID and b.DevType = '09' and a.MonItem = c.MonItem and c.MonType = '03' and a.DataYMD >= '%s' and a.DataYMD <= '%s' group by a.DevID, a.DataYMD, c.MonItemName",
		provId, dateList[0], dateList[days-1])
	rows, err = db.Query(sqllan)
	if err != nil {
		fmt.Println("获取灯诱数据异常:", err)
		return "", ""
	}
	sexDevDateDataMap := make(map[string]map[string]map[string]float64)
	sexDateDataMap := make(map[string]map[string]float64)
	sexMonitemSet := mapset.NewSet()
	for rows.Next() {
		var (
			DevID       sql.NullString
			DataYMD     sql.NullString
			MonItemName sql.NullString
			DevCount    sql.NullFloat64
		)
		err = rows.Scan(&DevID, &DataYMD, &MonItemName, &DevCount)
		if err != nil {
			fmt.Println("读取灯诱数据异常异常:", err)
			return "", ""
		}
		if _, ok := sexDevDateDataMap[DevID.String]; !ok {
			sexDevDateDataMap[DevID.String] = make(map[string]map[string]float64)
		}
		if _, ok := sexDevDateDataMap[DevID.String][DataYMD.String]; !ok {
			sexDevDateDataMap[DevID.String][DataYMD.String] = make(map[string]float64)
			sexDevDateDataMap[DevID.String][DataYMD.String]["sum"] = 0
		}
		if _, ok := sexDevDateDataMap[DevID.String][DataYMD.String][MonItemName.String]; !ok {
			sexDevDateDataMap[DevID.String][DataYMD.String][MonItemName.String] = 0
		}
		sexDevDateDataMap[DevID.String][DataYMD.String][MonItemName.String] += DevCount.Float64
		sexDevDateDataMap[DevID.String][DataYMD.String]["sum"] += DevCount.Float64

		if _, ok := sexDateDataMap[DataYMD.String]; !ok {
			sexDateDataMap[DataYMD.String] = make(map[string]float64)
		}
		if _, ok := sexDateDataMap[DataYMD.String][MonItemName.String]; !ok {
			sexDateDataMap[DataYMD.String][MonItemName.String] = 0
			sexDateDataMap[DataYMD.String]["sum"] = 0
		}
		sexDateDataMap[DataYMD.String][MonItemName.String] += DevCount.Float64
		sexDateDataMap[DataYMD.String]["sum"] += DevCount.Float64
		sexMonitemSet.Add(MonItemName.String)
	}

	xlsx := excelize.NewFile()

	curSheetName := "灯诱逐设备统计"
	err = xlsx.SetSheetName("Sheet1", curSheetName)
	if err != nil {
		fmt.Println("修改Sheet1名称错误")
	}

	// 生成灯诱统计
	lightDevColumnList := []eneity.Column{
		{"市", "CityName"},
		{"县", "AreaName"},
		{"监测点", "SiteName"},
		{"设备名称", "DevName"},
		{"设备类型", "DevTypeName"},
		{"设备编号", "DevID"},
		{"设备源编号", "SourceID"},
	}
	lightMonitemColumnList := []eneity.Column{
		{"该设备当日累计虫量", "sum"},
	}
	header := []interface{}{"市", "县", "监测点", "设备名称", "设备类型", "设备编号", "设备源编号", "日期", "该设备当日累计虫量"}
	for _, monitem := range lightMonitemSet.ToSlice() {
		lightMonitemColumnList = append(lightMonitemColumnList, eneity.Column{
			Name: monitem.(string), Prop: monitem.(string),
		})
		header = append(header, monitem.(string))
	}
	err = xlsx.SetSheetRow(curSheetName, "A1", &header)
	if err != nil {
		fmt.Println("生成灯诱单灯表头错误！")
		return "", ""
	}
	row := 2
	for _, dev := range lightDevList {
		for _, date := range dateList {
			var data []interface{}
			for _, col := range lightDevColumnList {
				data = append(data, dev[col.Prop])
			}
			data = append(data, date)
			for _, col := range lightMonitemColumnList {
				monitem := col.Prop
				if v, ok := lightDevDateDataMap[dev["DevID"]][date][monitem]; ok {
					data = append(data, v)
				} else {
					data = append(data, "/")
				}
			}
			err = xlsx.SetSheetRow(curSheetName, fmt.Sprintf("A%d", row), &data)
			if err != nil {
				fmt.Println("插入灯诱单灯错误！", err)
				return "", ""
			}
			row += 1
		}
	}

	// 生成性诱统计
	curSheetName = "性诱逐设备统计"
	_, err = xlsx.NewSheet(curSheetName)
	if err != nil {
		fmt.Println("新建性诱单灯Sheet错误！", err)
		return "", ""
	}
	sexDevColumnList := []eneity.Column{
		{"市", "CityName"},
		{"县", "AreaName"},
		{"监测点", "SiteName"},
		{"设备名称", "DevName"},
		{"设备类型", "DevTypeName"},
		{"设备编号", "DevID"},
		{"设备源编号", "SourceID"},
	}
	sexMonitemColumnList := []eneity.Column{
		{"该设备当日累计虫量", "sum"},
	}
	header = []interface{}{"市", "县", "监测点", "设备名称", "设备类型", "设备编号", "设备源编号", "日期", "该设备当日累计虫量"}
	for _, monitem := range sexMonitemSet.ToSlice() {
		sexMonitemColumnList = append(sexMonitemColumnList, eneity.Column{
			Name: monitem.(string), Prop: monitem.(string),
		})
		header = append(header, monitem.(string))
	}
	err = xlsx.SetSheetRow(curSheetName, "A1", &header)
	if err != nil {
		fmt.Println("生成性诱单灯表头错误！", err)
		return "", ""
	}
	row = 2
	for _, dev := range sexDevList {
		for _, date := range dateList {
			var data []interface{}
			for _, col := range sexDevColumnList {
				data = append(data, dev[col.Prop])
			}
			data = append(data, date)
			for _, col := range sexMonitemColumnList {
				monitem := col.Prop
				if v, ok := sexDevDateDataMap[dev["DevID"]][date][monitem]; ok {
					data = append(data, v)
				} else {
					data = append(data, "/")
				}
			}
			err = xlsx.SetSheetRow(curSheetName, fmt.Sprintf("A%d", row), &data)
			if err != nil {
				fmt.Println("插入性诱单灯错误！", err)
				return "", ""
			}
			row += 1
		}
	}

	// 逐日合计
	curSheetName = "全自治区逐日灯诱和性诱数据统计"
	_, err = xlsx.NewSheet(curSheetName)
	if err != nil {
		fmt.Println("新建逐日合计Sheet错误！", err)
		return "", ""
	}
	allMonitemSet := lightMonitemSet.Union(sexMonitemSet)
	allColumnList := []eneity.Column{
		{"当日总计", "sum"},
	}
	header = []interface{}{"日期", "当日总计", "性诱当日总计", "灯诱当日总计"}
	for _, monitem := range allMonitemSet.ToSlice() {
		allColumnList = append(allColumnList, eneity.Column{Name: monitem.(string), Prop: monitem.(string)})
		header = append(header, fmt.Sprintf("%s-性诱", monitem))
		header = append(header, fmt.Sprintf("%s-灯诱", monitem))
	}
	err = xlsx.SetSheetRow(curSheetName, "A1", &header)
	if err != nil {
		fmt.Println("生成逐日合计表头错误！", err)
		return "", ""
	}
	row = 2
	for _, date := range dateList {
		data := []interface{}{date}
		var sum float64
		if v, ok := sexDateDataMap[date]["sum"]; ok {
			sum += v
		}
		if v, ok := lightDateDataMap[date]["sum"]; ok {
			sum += v
		}
		data = append(data, sum)
		for _, col := range allColumnList {
			monitem := col.Prop
			if v, ok := sexDateDataMap[date][monitem]; ok {
				data = append(data, v)
			} else {
				data = append(data, "/")
			}
			if v, ok := lightDateDataMap[date][monitem]; ok {
				data = append(data, v)
			} else {
				data = append(data, "/")
			}
		}
		err = xlsx.SetSheetRow(curSheetName, fmt.Sprintf("A%d", row), &data)
		if err != nil {
			fmt.Println("插入逐日合计错误！", err)
			return "", ""
		}
		row += 1
	}
	filepath := fmt.Sprintf("./file/%s(%s~%s)灯诱及性诱日统计_%d.xlsx", static.Citys[provId], beginDate, endDate, time.Now().Unix())
	err = xlsx.SaveAs(filepath)
	if err != nil {
		fmt.Println("保存excel异常:", err)
		return "", ""
	}
	err = db.Close()
	if err != nil {
		fmt.Println("数据库连接关闭异常:", err)
		return "", ""
	}
	msg := fmt.Sprintf("%s设备(%s~%s)灯诱性诱日统计结果可见附件，请尽快完成设备排查并完成数据补录。", static.Citys[provId], beginDate, endDate)
	return filepath, msg
}
