package utils

import (
	"encoding/json"
	"fmt"
	"github.com/levigross/grequests"
	"pestiot/static"
	"strings"
)

func WechatSendMessage(message string, mentionedList *[]string, mentionedMobileList *[]string) {
	data := map[string]interface{}{
		"msgtype": "text",
		"text": map[string]interface{}{
			"content":               message,
			"mentioned_list":        &mentionedList,
			"mentioned_mobile_list": &mentionedMobileList,
		},
	}

	response, err := grequests.Post(static.WechatWebHook, &grequests.RequestOptions{
		JSON: data,
	})
	if err != nil {
		fmt.Println("推送消息失败：", err)
		return
	}

	fmt.Println("推送消息成功：", response.String())
}

func WechatFileUpload(filePath string, message string, mentionedList *[]string, mentionedMobileList *[]string) {
	fd, err := grequests.FileUploadFromDisk(filePath)
	if err != nil {
		fmt.Println(fmt.Sprintf("读取文件(%s)异常：", filePath), err)
		return
	}
	nameStrs := strings.Split(fd[0].FileName, "/")
	fd[0].FileName = nameStrs[len(nameStrs)-1]
	response, err := grequests.Post(static.WechatFileUploadUrl, &grequests.RequestOptions{
		Files: fd,
	})
	if err != nil {
		fmt.Println(fmt.Sprintf("上传文件(%s)异常：", filePath), err)
		return
	}

	var retJson map[string]interface{}
	err = json.Unmarshal(response.Bytes(), &retJson)
	if err != nil {
		fmt.Println(fmt.Sprintf("解析上传文件(%s)返回值异常：", filePath), err)
		return
	}
	mediaId := retJson["media_id"]

	data := map[string]interface{}{
		"msgtype": "file",
		"file": map[string]interface{}{
			"media_id":              mediaId,
			"mentioned_list":        mentionedList,
			"mentioned_mobile_list": mentionedMobileList,
		},
	}

	response, err = grequests.Post(static.WechatWebHook, &grequests.RequestOptions{
		JSON: data,
	})
	if err != nil {
		fmt.Println(fmt.Sprintf("推送文件(%s)异常：", filePath), err)
		return
	}

	fmt.Println("推送文件成功：", response.String())

	WechatSendMessage(message, mentionedList, nil)
}
